@@ -805,31 +805,22 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 		URL url;
 
 		// Copy
-		action = javax.swing.TransferHandler // JAVA13:
-				// org.jgraph.plaf.basic.TransferHandler
-				.getCopyAction();
+		action = javax.swing.TransferHandler.getCopyAction();
 		url = getClass().getClassLoader().getResource(
 				"org/jgraph/example/resources/copy.gif");
-		action.putValue(Action.SMALL_ICON, new ImageIcon(url));
-		toolbar.add(copy = new EventRedirector(action));
+		toolbar.add(copy = new EventRedirector(action, new ImageIcon(url)));
 
 		// Paste
-		action = javax.swing.TransferHandler // JAVA13:
-				// org.jgraph.plaf.basic.TransferHandler
-				.getPasteAction();
+		action = javax.swing.TransferHandler.getPasteAction();
 		url = getClass().getClassLoader().getResource(
 				"org/jgraph/example/resources/paste.gif");
-		action.putValue(Action.SMALL_ICON, new ImageIcon(url));
-		toolbar.add(paste = new EventRedirector(action));
+		toolbar.add(paste = new EventRedirector(action, new ImageIcon(url)));
 
 		// Cut
-		action = javax.swing.TransferHandler // JAVA13:
-				// org.jgraph.plaf.basic.TransferHandler
-				.getCutAction();
+		action = javax.swing.TransferHandler.getCutAction();
 		url = getClass().getClassLoader().getResource(
 				"org/jgraph/example/resources/cut.gif");
-		action.putValue(Action.SMALL_ICON, new ImageIcon(url));
-		toolbar.add(cut = new EventRedirector(action));
+		toolbar.add(cut = new EventRedirector(action, new ImageIcon(url)));
 
 		// Remove
 		URL removeUrl = getClass().getClassLoader().getResource(
@@ -952,8 +943,8 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 		protected Action action;
 
 		// Construct the "Wrapper" Action
-		public EventRedirector(Action a) {
-			super("", (ImageIcon) a.getValue(Action.SMALL_ICON));
+		public EventRedirector(Action a, ImageIcon icon) {
+			super("", icon);
 			this.action = a;
 		}
 
