@@ -1,7 +1,7 @@
 /*
- * @(#)JGraph.java
+ * $Id: JGraph.java,v 1.85 2008/10/02 14:37:27 gaudenz Exp $
  *
- * Copyright (c) 2001-2006 Gaudenz Alder
+ * Copyright (c) 2001-2008 Gaudenz Alder
  *
  */
 package org.jgraph;
@@ -41,8 +41,11 @@ import javax.swing.JViewport;
 import javax.swing.Scrollable;
 import javax.swing.SwingConstants;
 
+import org.jgraph.event.GraphModelEvent;
 import org.jgraph.event.GraphSelectionEvent;
 import org.jgraph.event.GraphSelectionListener;
+import org.jgraph.event.GraphLayoutCacheEvent.GraphLayoutCacheChange;
+import org.jgraph.event.GraphModelEvent.GraphModelChange;
 import org.jgraph.graph.AbstractCellView;
 import org.jgraph.graph.AttributeMap;
 import org.jgraph.graph.BasicMarqueeHandler;
@@ -177,13 +180,9 @@ import org.jgraph.plaf.basic.BasicGraphUI;
  * @version 2.1 16/03/03
  * 
  */
-public class JGraph
-//DO NOT REMOVE OR MODIFY THIS LINE!
-		extends JComponent // JAVA13:
-		// org.jgraph.plaf.basic.TransferHandler.JAdapterComponent
-		implements Scrollable, Accessible, Serializable {
+public class JGraph extends JComponent implements Scrollable, Accessible, Serializable {
 
-	public static final String VERSION = "JGraph (v5.10.1.5)";
+	public static final String VERSION = "JGraph (v5.12.2.1)";
 
 	public static final int DOT_GRID_MODE = 0;
 
@@ -242,9 +241,6 @@ public class JGraph
 	/** Off screen image for double buffering */
 	protected transient Image offscreen;
 
-	/** Whether or not the current background image is correct */
-	protected transient boolean offscreenValid = false;
-
 	/** The bounds of the offscreen buffer */
 	protected transient Rectangle2D offscreenBounds;
 	
@@ -253,6 +249,13 @@ public class JGraph
 
 	/** Graphics object of off screen image */
 	protected transient Graphics offgraphics;
+	
+	/** Whether or not the current background image is correct */
+	protected transient Rectangle2D offscreenDirty = null;
+	
+	protected transient boolean wholeOffscreenDirty = false;
+	
+	protected transient double wholeOffscreenDirtyProportion = 0.8;
 
 	/**
 	 * The buffer around the offscreen graphics object that provides the
@@ -266,6 +269,9 @@ public class JGraph
 	 * buffering. Volatile 
 	 */
 	protected boolean volatileOffscreen = false;
+	
+	/** Stores whether the last double buffer allocation worked or not */
+	protected boolean lastBufferAllocated = true;
 
 	/** Holds the background image. */
 	protected ImageIcon backgroundImage;
@@ -315,6 +321,9 @@ public class JGraph
 	/** True if the ports are scaled. Bound property. */
 	protected boolean portsScaled = true;
 
+	/** True if port are painted above all other cells. */
+	protected boolean portsOnTop = true;
+	
 	/** True if the graph allows to move cells below zero. */
 	protected boolean moveBelowZero = false;
 	
@@ -1311,7 +1320,7 @@ public class JGraph
 	 * Returns true if the graph accepts drops/pastes from external sources.
 	 */
 	public boolean isXorEnabled() {
-		return xorEnabled;
+		return (xorEnabled && isOpaque());
 	}
 
 	/**
@@ -1741,6 +1750,7 @@ public class JGraph
 			scale = newValue;
 			boolean zoomIn = true;
 			Rectangle newView = null;
+			clearOffscreen();
 			if (view != null) {
 				double scaleRatio = newValue / oldValue;
 				int newCenterX = (int) (center.getX() * scaleRatio);
@@ -1757,7 +1767,6 @@ public class JGraph
 					zoomIn = false;
 				}
 			}
-			offscreenValid = false;
 			firePropertyChange(SCALE_PROPERTY, oldValue, newValue);
 			// When zooming in, do it after the revalidation otherwise
 			// it intermittently moves to the old co-ordinate system
@@ -1767,6 +1776,21 @@ public class JGraph
 		}
 	}
 
+	/**
+	 * Invalidate the offscreen region, do not just delete it, since if the new
+	 * region is smaller than the old you may not wish to re-create the buffer
+	 */
+	public void clearOffscreen() {
+
+		if (offscreen != null) {
+			int h = offscreen.getHeight(this);
+			int w = offscreen.getWidth(this);
+			Rectangle2D dirtyRegion = new Rectangle2D.Double(0, 0, w, h);
+			fromScreen(dirtyRegion);
+			addOffscreenDirty(dirtyRegion);
+		}
+	}
+
 	/**
 	 * Returns the center of the component relative to the parent viewport's
 	 * position.
@@ -1858,6 +1882,10 @@ public class JGraph
 	public void setGridVisible(boolean flag) {
 		boolean oldValue = gridVisible;
 		gridVisible = flag;
+		// Clear the double buffer if the grid has been enabled
+		if (flag && !oldValue) {
+			clearOffscreen();
+		}
 		firePropertyChange(GRID_VISIBLE_PROPERTY, oldValue, flag);
 	}
 
@@ -1901,6 +1929,14 @@ public class JGraph
 		firePropertyChange(PORTS_SCALED_PROPERTY, oldValue, flag);
 	}
 
+	public boolean isPortsOnTop() {
+		return portsOnTop;
+	}
+
+	public void setPortsOnTop(boolean portsOnTop) {
+		this.portsOnTop = portsOnTop;
+	}
+
 	/**
 	 * Returns true if the graph will be anti aliased.
 	 * 
@@ -2075,6 +2111,24 @@ public class JGraph
 			}
 			offscreen = null;
 			offgraphics = null;
+			Runtime runtime = Runtime.getRuntime();
+			long maxMemory = runtime.maxMemory();
+			long allocatedMemory = runtime.totalMemory();
+			long freeMemory = runtime.freeMemory();
+			long totalFreeMemory = (freeMemory + (maxMemory - allocatedMemory)) / 1024;
+			// Calculate size of buffer required (assuming TYPE_INT_RGB which
+			// stores each pixel in a 32-bit int )
+			long memoryRequired = width*height*4/1024;
+			if (memoryRequired > totalFreeMemory) {
+				if (lastBufferAllocated) {
+					// If the last attempt to allocate a buffer worked it might
+					// be we need to reclaim the memory before the next one
+					// will work
+					System.gc();
+				}
+				lastBufferAllocated = false;
+				return null;
+			}
 			if (offscreen == null && volatileOffscreen) {
 				try {
 					offscreen = createVolatileImage(width, height);
@@ -2093,8 +2147,10 @@ public class JGraph
 				// of it, might also be faster to calculate in
 				// advance whether they is enough memory to create image
 				// rather than let it try and throw error.
+				lastBufferAllocated = false;
 				return null;
 			}
+			lastBufferAllocated = true;
 			setupOffScreen(x, y, width, height, newOffscreenBuffer);
 		} else if (offscreen instanceof VolatileImage) {
 			int valCode = ((VolatileImage) offscreen)
@@ -2118,16 +2174,32 @@ public class JGraph
 				}
 				setupOffScreen(x, y, width, height, newOffscreenBuffer);
 			} else if (valCode == VolatileImage.IMAGE_RESTORED) {
-				setOffscreenValid(false);
+				addOffscreenDirty(new Rectangle2D.Double(0, 0, getWidth(), getHeight()));
 			}
 		}
-		if (!isOffscreenValid()) {
-			offgraphics.setColor(getBackground());
-			offgraphics.setPaintMode();
-			offgraphics.fillRect(0, 0, (int) graphBounds.getWidth(), (int) graphBounds
-					.getHeight());
-			((BasicGraphUI)getUI()).drawGraph(offgraphics, null);
-			offscreenValid = true;
+		Rectangle2D offscreenDirty = getOffscreenDirty();
+		if (offscreenDirty != null) {
+			if (isOpaque()) {
+				offgraphics.setColor(getBackground());
+				offgraphics.setPaintMode();
+			} else {
+				((Graphics2D) offgraphics).setComposite(AlphaComposite.getInstance(
+					      AlphaComposite.CLEAR, 0.0f));
+			}
+			toScreen(offscreenDirty);
+			offscreenDirty.setRect(offscreenDirty.getX()
+					- (getHandleSize() + 1), offscreenDirty.getY()
+					- (getHandleSize() + 1), offscreenDirty.getWidth()
+					+ (getHandleSize() + 1) * 2, offscreenDirty.getHeight()
+					+ (getHandleSize() + 1) * 2);
+			offgraphics.fillRect((int) offscreenDirty.getX(),
+					(int) offscreenDirty.getY(), (int) offscreenDirty
+							.getWidth(), (int) offscreenDirty.getHeight());
+			if (!isOpaque()) {
+				((Graphics2D) offgraphics).setComposite(AlphaComposite.SrcOver);
+			}
+			((BasicGraphUI) getUI()).drawGraph(offgraphics, offscreenDirty);
+			clearOffscreenDirty();
 		}
 		return offgraphics;
 	}
@@ -2141,8 +2213,9 @@ public class JGraph
 		GraphicsConfiguration graphicsConfig = getGraphicsConfiguration();
 		if (graphicsConfig != null) {
 			try {
-				offscreen = graphicsConfig.createCompatibleImage(width,
-						height, Transparency.OPAQUE);
+				offscreen = graphicsConfig.createCompatibleImage(width, height,
+						(isOpaque()) ? Transparency.OPAQUE
+								: Transparency.TRANSLUCENT);
 			} catch (OutOfMemoryError e) {
 				offscreen = null;
 				offgraphics = null;
@@ -2150,7 +2223,8 @@ public class JGraph
 		} else {
 			try {
 				offscreen = new BufferedImage(width, height,
-						BufferedImage.TYPE_INT_RGB);
+						isOpaque() ? BufferedImage.TYPE_INT_RGB
+								: BufferedImage.TYPE_INT_ARGB);
 			} catch (OutOfMemoryError e) {
 				offscreen = null;
 				offgraphics = null;
@@ -2168,12 +2242,22 @@ public class JGraph
 	 */
 	protected void setupOffScreen(int x, int y, int width, int height, Rectangle2D newOffscreenBuffer) {
 		offgraphics = offscreen.getGraphics();
-		setOffscreenValid(false);
-		offgraphics.setColor(getBackground());
-		offgraphics.setPaintMode();
+		if (isOpaque()) {
+			offgraphics.setColor(getBackground());
+			offgraphics.setPaintMode();
+		} else {
+			((Graphics2D) offgraphics).setComposite(AlphaComposite.getInstance(
+					AlphaComposite.CLEAR, 0.0f));
+		}
 		offgraphics.fillRect(0, 0, width, height);
+		if (!isOpaque()) {
+			 ((Graphics2D) offgraphics).setComposite(AlphaComposite.SrcOver);
+		}
+		((BasicGraphUI)getUI()).drawGraph(offgraphics, null);
 		offscreenBounds = newOffscreenBuffer;
 		offscreenOffset = new Point2D.Double(x, y);
+		// Clear the offscreen, we've just drawn the whole thing
+		clearOffscreenDirty();
 	}
 
 	/**
@@ -2183,6 +2267,48 @@ public class JGraph
 		return offscreen;
 	}
 
+	/**
+	 * Returns the area that is deemed dirty for the next double buffered redraw
+	 * 
+	 * @return the area that is deemed dirty for the next double buffered redraw
+	 */
+	public Rectangle2D getOffscreenDirty() {
+		return offscreenDirty;
+	}
+
+	/**
+	 * Adds the specified area to the region deemed dirty for the next double
+	 * buffered redraw
+	 * 
+	 * @param offscreenDirty
+	 *            the region to add
+	 */
+	public void addOffscreenDirty(Rectangle2D offscreenDirty) {
+		if (this.offscreenDirty == null && offscreenDirty != null) {
+			this.offscreenDirty = (Rectangle2D) offscreenDirty.clone();
+		} else if (offscreenDirty != null) {
+			this.offscreenDirty.add(offscreenDirty);
+		}
+	}
+
+	/**
+	 * Clears the region deemed dirty for the next double buffered redraw
+	 */
+	public void clearOffscreenDirty() {
+		offscreenDirty = null;
+	}
+	
+	/**
+	 * Schedules the offscreen resources taken by the offscreen buffer to
+	 * be reclaimed. Note that this does not force garbage collection
+	 */
+	public void releaseOffscreenResources() {
+		offscreen.flush();
+		offgraphics.dispose();
+		offscreen = null;
+		offgraphics = null;
+	}
+	
 	/**
 	 * Utility method to draw the off screen buffer
 	 * 
@@ -2226,6 +2352,7 @@ public class JGraph
 		return getGraphics().drawImage(offscreen, rect.x, rect.y,
 				rect.x + rect.width, rect.y + rect.height, rect.x, rect.y,
 				rect.x + rect.width, rect.y + rect.height, this);
+
 	}
 
 	/**
@@ -2247,9 +2374,18 @@ public class JGraph
 	public void setBackgroundImage(ImageIcon backgroundImage) {
 		ImageIcon oldValue = this.backgroundImage;
 		this.backgroundImage = backgroundImage;
+		clearOffscreen();
 		firePropertyChange(PROPERTY_BACKGROUNDIMAGE, oldValue, backgroundImage);
 	}
 
+	/**
+	 * Override parent to clear offscreen double buffer
+	 */
+	public void setBackground(Color bg) {
+		clearOffscreen();
+		super.setBackground(bg);
+	}
+
 	/**
 	 * @return the backgroundScaled
 	 */
@@ -2257,21 +2393,6 @@ public class JGraph
 		return backgroundScaled;
 	}
 
-	/**
-	 * @return the offscreenValid
-	 */
-	public boolean isOffscreenValid() {
-		return offscreenValid;
-	}
-
-	/**
-	 * @param offscreenValid
-	 *            the offscreenValid to set
-	 */
-	public void setOffscreenValid(boolean offscreenValid) {
-		this.offscreenValid = offscreenValid;
-	}
-
 	/**
 	 * @return the offscreenOffset
 	 */
@@ -2320,9 +2441,23 @@ public class JGraph
 	 *            the backgroundComponent to set
 	 */
 	public void setBackgroundComponent(Component backgroundComponent) {
+		clearOffscreen();
 		this.backgroundComponent = backgroundComponent;
 	}
 
+	/*
+	 * Overriden to change painting style for opaque components
+	 * @see javax.swing.JComponent#setOpaque(boolean)
+	 */
+	public void setOpaque(boolean opaque) {
+		// Due to problems with XOR painting on transparent backgrounds
+		// switch off XOR for non-opaque components
+		if (!opaque) {
+			setXorEnabled(false);
+		}
+		super.setOpaque(opaque);
+	}
+
 	/**
 	 * Returns the <code>GraphModel</code> that is providing the data.
 	 * 
@@ -2345,6 +2480,7 @@ public class JGraph
 	public void setModel(GraphModel newModel) {
 		GraphModel oldModel = graphModel;
 		graphModel = newModel;
+		clearOffscreen();
 		firePropertyChange(GRAPH_MODEL_PROPERTY, oldModel, graphModel);
 		// FIX: Use Listener
 		if (graphLayoutCache != null
@@ -2377,13 +2513,22 @@ public class JGraph
 	 *            view-data
 	 */
 	public void setGraphLayoutCache(GraphLayoutCache newLayoutCache) {
+		if (!isSelectionEmpty())
+		{
+			clearSelection();
+		}
 		GraphLayoutCache oldLayoutCache = graphLayoutCache;
 		graphLayoutCache = newLayoutCache;
+		clearOffscreen();
 		firePropertyChange(GRAPH_LAYOUT_CACHE_PROPERTY, oldLayoutCache,
 				graphLayoutCache);
 		if (graphLayoutCache != null
-				&& graphLayoutCache.getModel() != getModel())
+				&& graphLayoutCache.getModel() != getModel()) {
 			setModel(graphLayoutCache.getModel());
+		} else {
+			// Forces an update of the layout cache internal state (ports field)
+			graphLayoutCache.setModel(getModel());
+		}
 		invalidate();
 	}
 
@@ -2852,56 +2997,6 @@ public class JGraph
 		repaint();
 	}
 
-	/**
-	 * You should not call this method directly on a JGraph if you are using
-	 * double buffering.
-	 * 
-	 * @see javax.swing.JComponent#repaint(long, int, int, int, int)
-	 */
-	public void repaint(long tm, int x, int y, int width, int height) {
-		super.repaint(tm, x, y, width, height);
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.JComponent#repaint(java.awt.Rectangle)
-	 */
-	public void repaint(Rectangle r) {
-		offscreenValid = false;
-		super.repaint(r);
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see java.awt.Component#repaint()
-	 */
-	public void repaint() {
-		offscreenValid = false;
-		super.repaint();
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see java.awt.Component#repaint(int, int, int, int)
-	 */
-	public void repaint(int x, int y, int width, int height) {
-		offscreenValid = false;
-		super.repaint(x, y, width, height);
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see java.awt.Component#repaint(long)
-	 */
-	public void repaint(long tm) {
-		offscreenValid = false;
-		super.repaint(tm);
-	}
-
 	// /* (non-Javadoc)
 	// * @see javax.swing.JComponent#isOptimizedDrawingEnabled()
 	// */
@@ -2960,6 +3055,44 @@ public class JGraph
 		return null;
 	}
 
+	/**
+	 * Calculates the clip 
+	 * @param change
+	 * @return the total region dirty as a result of this change
+	 */
+	public Rectangle2D getClipRectangle(GraphLayoutCacheChange change) {
+		List removed = DefaultGraphModel.getDescendants(getModel(), change.getRemoved());
+		Rectangle2D removedBounds = (removed != null && !removed.isEmpty()) ? getCellBounds(removed.toArray()) : null;
+		List inserted = DefaultGraphModel.getDescendants(getModel(), change.getInserted());
+		Rectangle2D insertedBounds = (inserted != null && !inserted.isEmpty()) ? getCellBounds(inserted.toArray()) : null;
+		List changed = DefaultGraphModel.getDescendants(getModel(), change.getChanged());
+		Rectangle2D changedBounds = (changed != null && !changed.isEmpty()) ? getCellBounds(changed.toArray()) : null;
+		List context = DefaultGraphModel.getDescendants(getModel(), change.getContext());
+		Rectangle2D contextBounds = (context != null && !context.isEmpty()) ? getCellBounds(context.toArray()) : null;
+
+		Rectangle2D clip = removedBounds;
+
+		if (clip == null) {
+			clip = insertedBounds;
+		} else if (insertedBounds != null) {
+			clip.add(insertedBounds);
+		}
+
+		if (clip == null) {
+			clip = changedBounds;
+		} else if (changedBounds != null) {
+			clip.add(changedBounds);
+		}
+
+		if (clip == null) {
+			clip = contextBounds;
+		} else if (contextBounds != null) {
+			clip.add(contextBounds);
+		}
+
+		return clip;
+	}
+
 	/**
 	 * Serialization support.
 	 */
