@@ -1,11 +1,12 @@
 /*
- * @(#)DefaultGraphModel.java	1.0 03-JUL-04
+ * $Id: DefaultGraphModel.java,v 1.24 2008/11/12 23:33:37 david Exp $
  * 
- * Copyright (c) 2001-2004 Gaudenz Alder
+ * Copyright (c) 2001-2008 Gaudenz Alder
  *  
  */
 package org.jgraph.graph;
 
+import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.Serializable;
@@ -36,7 +37,7 @@ import org.jgraph.event.GraphModelEvent;
 import org.jgraph.event.GraphModelListener;
 
 /**
- * A simple implementation of a graph model.
+ * The default implementation of a graph model.
  * 
  * @version 1.0 1/1/02
  * @author Gaudenz Alder
@@ -1035,6 +1036,13 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 					return null;
 				}
 
+				public Rectangle2D getDirtyRegion() {
+					return null;
+				}
+
+				public void setDirtyRegion(Rectangle2D dirty) {
+				}
+
 			});
 		}
 	}
@@ -1096,6 +1104,9 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 
 		/* Parent map for the next execution. */
 		protected ParentMap parentMap, previousParentMap;
+		
+		/** The dirty region of the change prior to it happening */
+		protected Rectangle2D dirtyRegion = null;
 
 		/* ConnectionSet for the next execution. */
 		protected ConnectionSet connectionSet, previousConnectionSet;
@@ -1278,6 +1289,14 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 			return previousParentMap;
 		}
 
+		public Rectangle2D getDirtyRegion() {
+			return dirtyRegion;
+		}
+
+		public void setDirtyRegion(Rectangle2D dirty) {
+			this.dirtyRegion = dirty;
+		}
+		
 		/**
 		 * Redoes a change.
 		 * 
@@ -1305,6 +1324,7 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 		 * invert the last execution.
 		 */
 		public void execute() {
+//			dirtyRegion = null;
 			// Compute Changed Cells
 			Set tmp = new HashSet();
 			if (attributes != null)
@@ -1376,7 +1396,6 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 				s += "No parent map\n";
 			return s;
 		}
-
 	}
 
 	/**
@@ -1494,6 +1513,13 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 			return null;
 		}
 
+		public Rectangle2D getDirtyRegion() {
+			return null;
+		}
+
+		public void setDirtyRegion(Rectangle2D dirty) {
+		}
+
 		/**
 		 * Allows a <code>GraphLayoutCache</code> to add and execute and
 		 * UndoableEdit in this change. This does also work if the parent edit
@@ -1841,7 +1867,7 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 	 * set. This can be used on vertices, edges and ports.
 	 */
 	public static Set getEdges(GraphModel model, Object[] cells) {
-		Set result = new HashSet();
+		Set result = new LinkedHashSet();
 		if (cells != null) {
 			// We know the minimum initial capacity of this set is cells.length
 			// We assume the cell has one port at a minimum
@@ -1851,7 +1877,12 @@ public class DefaultGraphModel extends UndoableEditSupport implements
 				allCells.add(cells[i]);
 			}
 			// Include descendants
-			allCells.addAll(getDescendants(model, cells));
+			List descendants = getDescendants(model, cells);
+			// Iterate through the list rather than adding all to preserve order
+			Iterator desIter = descendants.iterator();
+			while (desIter.hasNext()) {
+				allCells.add(desIter.next());
+			}
 			if (allCells != null) {
 				Iterator it = allCells.iterator();
 				while (it.hasNext()) {
