@@ -1,5 +1,5 @@
 /*
- * $Id: EdgeRenderer.java,v 1.18 2008/05/08 12:57:35 david Exp $ * 
+ * $Id: EdgeRenderer.java,v 1.22 2008/11/13 00:14:19 david Exp $ * 
  * Copyright (c) 2001-2007 Gaudenz Alder
  *  
  * See LICENSE file in distribution for licensing details of this source file
@@ -86,6 +86,9 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 	/** Width of the current edge view */
 	protected transient float lineWidth;
 
+	/** Cached value of whether the label is to be displayed */
+	transient protected boolean labelsEnabled;
+
 	/**
 	 * Boolean attributes of the current edgeview. Fill flags are checked for
 	 * valid decorations.
@@ -196,15 +199,15 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 			// If we have two control points, we can get rid of hit
 			// detection on do an intersection test on the two diagonals
 			// of rect and the line between the two points
+			Graphics2D g2 = (Graphics2D) graph.getGraphics();
 			EdgeView edgeView = (EdgeView) value;
-			if (edgeView.getPointCount() == 2) {
+			if (g2 == null || edgeView.getPointCount() == 2) {
 				Point2D p0 = edgeView.getPoint(0);
 				Point2D p1 = edgeView.getPoint(1);
 				if (rect.intersectsLine(p0.getX(), p0.getY(), p1.getX(), p1
 						.getY()))
 					return true;
-			} else {
-				Graphics2D g2 = (Graphics2D) graph.getGraphics();
+			} else if(g2 != null) {
 				if (g2.hit(rect, view.getShape(), true))
 					return true;
 			}
@@ -439,7 +442,7 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		Rectangle2D tmp = getPaintBounds(view);
 		int unit = GraphConstants.PERMILLE;
 		Point2D p0 = view.getPoint(0);
-		if (pos != null && tmp != null) {
+		if (pos != null && tmp != null && p0 != null) {
 			if (!isLabelTransformEnabled()) {
 				return view.getAbsoluteLabelPositionFromRelative(pos);
 			} else {
@@ -448,6 +451,10 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 				double dy = vector.getY();
 				double len = Math.sqrt(dx * dx + dy * dy);
 				if (len > 0) {
+					int pointIndex = view.getFirstPointOfSegment();
+					if (pointIndex >=0 && pointIndex < view.getPointCount()-1) {
+						p0 = view.getPoint(pointIndex);
+					}
 					double x = p0.getX() + (dx * pos.getX() / unit);
 					double y = p0.getY() + (dy * pos.getX() / unit);
 					x += (-dy * pos.getY() / len);
@@ -530,6 +537,7 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		Color tmp = GraphConstants.getForeground(map);
 		fontColor = (tmp != null) ? tmp : getForeground();
 		labelTransformEnabled = GraphConstants.isLabelAlongEdge(map);
+		labelsEnabled = GraphConstants.isLabelEnabled(map);
 	}
 
 	protected boolean isFillable(int decoration) {
@@ -657,7 +665,7 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 	 */
 	protected void paintLabel(Graphics g, String label, Point2D p,
 			boolean mainLabel) {
-		if (p != null && label != null && label.length() > 0 && metrics != null) {
+		if (labelsEnabled && p != null && label != null && label.length() > 0 && metrics != null) {
 			int sw = metrics.stringWidth(label);
 			int sh = metrics.getHeight();
 			Graphics2D g2 = (Graphics2D) g;
