@@ -44,7 +44,7 @@ public class EdgeView extends AbstractCellView {
 
 	/** Renderer for the class. */
 	public static transient EdgeRenderer renderer = new EdgeRenderer();
-
+	
 	/** List of points of the edge. May contain ports. */
 	protected List points;
 
@@ -547,6 +547,57 @@ public class EdgeView extends AbstractCellView {
 		}
 	}
 
+	/**
+	 * Utility method that returns the first point of the pair that forms the
+	 * segment that is relativeX along the edge as a proportion
+	 * 
+	 * @return the index of the first point. A value of -1 indicate to use the
+	 *         first and last points
+	 */
+	public int getFirstPointOfSegment() {
+		boolean exactSegment = GraphConstants
+				.isExactSegmentLabel(allAttributes);
+		double dx = 0;
+		double dy = 0;
+		int n = getPointCount();
+		if (exactSegment) {
+			// Determine the vector based on the actual edge segment the
+			// label lies on
+			Point2D lastPoint = getPoint(0);
+			double totalLength = 0;
+
+			for (int i = 1; i < n; i++) {
+				Point2D currentPoint = getPoint(i);
+				dx = currentPoint.getX() - lastPoint.getX();
+				dy = currentPoint.getY() - lastPoint.getY();
+				totalLength += Math.sqrt(dx * dx + dy * dy);
+				lastPoint = currentPoint;
+			}
+
+			double relativeX = getLabelPosition().getX()/(double)GraphConstants.PERMILLE;
+			double labelXPositionDistance = relativeX * totalLength;
+			totalLength = 0;
+			lastPoint = getPoint(0);
+			if (relativeX <= 0.0 || relativeX >= 1.0) {
+				return -1;
+			} else {
+				for (int i = 1; i < n; i++) {
+					Point2D currentPoint = getPoint(i);
+					dx = currentPoint.getX() - lastPoint.getX();
+					dy = currentPoint.getY() - lastPoint.getY();
+					totalLength += Math.sqrt(dx * dx + dy * dy);
+					if (totalLength > labelXPositionDistance) {
+						return i-1;
+					}
+				}
+			}
+		}
+		else {
+			return -1;
+		}
+		return -1;
+	}
+	
 	/**
 	 * Hook to return the vector that is taken as the base vector to compute
 	 * relative label positions. Normally, the vector goes from the first to the
@@ -559,8 +610,6 @@ public class EdgeView extends AbstractCellView {
 			double dx = 0;
 			double dy = 0;
 			// Finds an average distance
-			dx = 0;
-			dy = 0;
 			int n = getPointCount();
 			if (isLoop()) {
 				for (int i = 1; i < n; i++) {
@@ -573,10 +622,48 @@ public class EdgeView extends AbstractCellView {
 				dy /= n;
 				labelVector = new Point2D.Double(dx, dy);
 			} else {
-				Point2D point = getPoint(n - 1);
-				dx += point.getX() - p0.getX();
-				dy += point.getY() - p0.getY();
-				labelVector = new Point2D.Double(dx, dy);
+				boolean exactSegment = GraphConstants
+						.isExactSegmentLabel(allAttributes);
+				if (exactSegment) {
+					// Determine the vector based on the actual edge segment the
+					// label lies on
+					Point2D lastPoint = getPoint(0);
+					double totalLength = 0;
+
+					for (int i = 1; i < n; i++) {
+						Point2D currentPoint = getPoint(i);
+						dx = currentPoint.getX() - lastPoint.getX();
+						dy = currentPoint.getY() - lastPoint.getY();
+						totalLength += Math.sqrt(dx * dx + dy * dy);
+						lastPoint = currentPoint;
+					}
+
+					double relativeX = getLabelPosition().getX()/(double)GraphConstants.PERMILLE;
+					double labelXPositionDistance = relativeX * totalLength;
+					totalLength = 0;
+					lastPoint = getPoint(0);
+					if (relativeX <= 0.0 || relativeX >= 1.0) {
+						exactSegment = false;
+					} else {
+						for (int i = 1; i < n; i++) {
+							Point2D currentPoint = getPoint(i);
+							dx = currentPoint.getX() - lastPoint.getX();
+							dy = currentPoint.getY() - lastPoint.getY();
+							totalLength += Math.sqrt(dx * dx + dy * dy);
+							if (totalLength > labelXPositionDistance) {
+								labelVector = new Point2D.Double(dx, dy);
+								break;
+							}
+							lastPoint = currentPoint;
+						}
+					}
+				} 
+				if (!exactSegment || labelVector == null) {
+					Point2D point = getPoint(n - 1);
+					dx = point.getX() - p0.getX();
+					dy = point.getY() - p0.getY();
+					labelVector = new Point2D.Double(dx, dy);
+				}
 			}
 		}
 		return labelVector;
@@ -642,14 +729,13 @@ public class EdgeView extends AbstractCellView {
 	 * @return the absolute label position
 	 */
 	protected Point2D convertRelativeLabelPositionToAbsolute(Point2D geometry) {
-		int pointCount = getPointCount();
-		
-		double length = 0;
-		double[] segments = new double[pointCount];
 		Point2D pt = getPoint(0);
 		
 		if (pt != null)
 		{
+			double length = 0;
+			int pointCount = getPointCount();
+			double[] segments = new double[pointCount];
 			// Find the total length of the segments and also store the length
 			// of each segment
 			for (int i = 1; i < pointCount; i++)
@@ -1390,6 +1476,7 @@ public class EdgeView extends AbstractCellView {
 			Point2D storedRelativePosition = edge.convertRelativeLabelPositionToAbsolute(result);
 			if (p.equals(storedRelativePosition)) {
 				GraphConstants.setRemoveAttributes(edge.getAllAttributes(), new Object[] {GraphConstants.OFFSET});
+				edge.getAllAttributes().remove(GraphConstants.OFFSET);
 			} else {
 				Point2D off = new Point2D.Double(p.getX() - storedRelativePosition.getX(), p.getY() - storedRelativePosition.getY());
 				GraphConstants.setOffset(edge.getAllAttributes(), off);
