@@ -1,7 +1,7 @@
 /*
- * @(#)GraphLayoutCache.java 1.0 03-JUL-04
+ * $Id: GraphLayoutCache.java,v 1.39 2008/09/11 07:38:38 gaudenz Exp $
  * 
- * Copyright (c) 2001-2004 Gaudenz Alder
+ * Copyright (c) 2001-2008 Gaudenz Alder
  *  
  */
 package org.jgraph.graph;
@@ -32,13 +32,14 @@ import javax.swing.undo.UndoableEdit;
 import org.jgraph.event.GraphLayoutCacheEvent;
 import org.jgraph.event.GraphLayoutCacheListener;
 import org.jgraph.event.GraphModelEvent;
+import org.jgraph.util.RectUtils;
 
 /**
  * An object that defines the view of a graphmodel. This object maps between
  * model cells and views and provides a set of methods to change these views.
  * The view may also contain its own set of attributes and is therefore an
  * extension of an Observable, which may be observed by the GraphUI. It uses the
- * model to send its changes to the command history.
+ * model to send its changes to the command history. 
  * 
  * @version 1.0 1/1/02
  * @author Gaudenz Alder
@@ -68,9 +69,9 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 	 * Boolean indicating whether edited cells should be made visible if they
 	 * are changed via
 	 * {@link #edit(Map, ConnectionSet, ParentMap, UndoableEdit[])}. Default is
-	 * true.
+	 * false.
 	 */
-	protected boolean showsInvisibleEditedCells = true;
+	protected boolean showsInvisibleEditedCells = false;
 
 	/**
 	 * Boolean indicating whether inserted should be made visible if they are
@@ -158,7 +159,7 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 	protected boolean reconnectsEdgesToVisibleParent = false;
 
 	/**
-	 * The list of listeners that listen to the model.
+	 * The list of listeners that listen to the GraphLayoutCache.
 	 */
 	protected EventListenerList listenerList = new EventListenerList();
 
@@ -347,6 +348,12 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 							return null;
 						}
 
+						public Rectangle2D getDirtyRegion() {
+							return null;
+						}
+
+						public void setDirtyRegion(Rectangle2D dirty) {}
+
 					});
 		}
 	}
@@ -385,6 +392,7 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 				.getListeners(GraphLayoutCacheListener.class);
 	}
 
+
 	//
 	// Accessors
 	//
@@ -437,10 +445,10 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 	 */
 	public static Rectangle2D getBounds(CellView[] views) {
 		if (views != null && views.length > 0) {
-			Rectangle2D r = views[0].getBounds();
+			Rectangle2D r = (views[0] != null) ? views[0].getBounds() : null;
 			Rectangle2D ret = (r != null) ? (Rectangle2D) r.clone() : null;
 			for (int i = 1; i < views.length; i++) {
-				r = views[i].getBounds();
+				r = (views[i] != null) ? views[i].getBounds() : null;
 				if (r != null) {
 					if (ret == null)
 						ret = (r != null) ? (Rectangle2D) r.clone() : null;
@@ -1022,12 +1030,31 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 	 */
 	public void setVisible(Object[] visible, Object[] invisible,
 			ConnectionSet cs) {
-		GraphLayoutCacheEdit edit = new GraphLayoutCacheEdit(null, null,
-				visible, invisible);
-		edit.end();
-		graphModel.edit(null, cs, null, new UndoableEdit[] { edit });
+		setVisible(visible, invisible, null, cs);
 	}
 
+	/**
+	 * Changes the visibility state of the cells passed in. Note that the arrays
+	 * must contain cells, not cell views.
+	 * 
+	 * @param visible
+	 *            cells to be made visible
+	 * @param invisible
+	 *            cells to be made invisible
+	 * @param attributes
+	 *            a nested attribute map of cells/attribute maps
+	 * @param cs
+	 *            a <code>ConnectionSet</code> describing the new state of
+	 *            edge connections in the graph
+	 */
+	public void setVisible(Object[] visible, Object[] invisible,
+			Map attributes, ConnectionSet cs) {
+		GraphLayoutCacheEdit edit = new GraphLayoutCacheEdit(null, attributes,
+				visible, invisible);
+		edit.end();
+		graphModel.edit(attributes, cs, null, new UndoableEdit[] { edit });
+	}
+	
 	// This is used to augment the array passed to the setVisible method.
 	protected Object[] addVisibleDependencies(Object[] cells, boolean visible) {
 		if (cells != null) {
@@ -2007,7 +2034,7 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 	}
 
 	/**
-	 * An implementation of GraphViewChange.
+	 * An implementation of GraphLayoutCacheChange.
 	 */
 	public class GraphLayoutCacheEdit extends CompoundEdit implements
 			GraphLayoutCacheEvent.GraphLayoutCacheChange {
@@ -2019,6 +2046,11 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 		protected Map attributes, previousAttributes;
 
 		protected Object[] visible, invisible;
+		
+		/**
+		 * The dirty region associated with this event prior to the change
+		 */
+		protected Rectangle2D dirtyRegion = null;
 
 		// Remember which cells have changed for finding their context
 		protected Set changedCells = new HashSet();
@@ -2116,6 +2148,15 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 			return previousAttributes;
 		}
 
+		public Rectangle2D getDirtyRegion() {
+			return dirtyRegion;
+		}
+
+		public void setDirtyRegion(Rectangle2D dirty) {
+			this.dirtyRegion = dirty;
+			
+		}
+		
 		/**
 		 * Redoes a change.
 		 * 
@@ -2150,6 +2191,15 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 				for (int i = 0; i < hidden.length; i++)
 					if (hidden[i] != null)
 						mapping.put(hidden[i].getCell(), hidden[i]);
+			
+			if (invisible != null && invisible.length >0) {
+				CellView[] invisibleViews = new CellView[invisible.length];
+				invisibleViews = getMapping(invisible, true);
+				Rectangle2D changedBounds = getBounds(invisibleViews);
+				dirtyRegion = RectUtils.union(dirtyRegion, changedBounds);
+			}
+
+						
 			if (!remembersCellViews) // already remembered
 				hidden = getMapping(invisible);
 			// Handle visibility
@@ -2175,7 +2225,6 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 			if (attributes != null) {
 				previousAttributes = attributes;
 				changedCells.addAll(attributes.keySet());
-				attributes = handleAttributes(attributes);
 			}
 			if (updatePorts)
 				updatePorts();
@@ -2190,13 +2239,21 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 				}
 			}
 			changedCells.addAll(parentSet);
+			Set ctx = DefaultGraphModel.getEdges(getModel(), changedCells
+					.toArray());
+			context = getMapping(ctx.toArray());
+			Set allChangedCells = new HashSet(changedCells);
+			allChangedCells.addAll(ctx);
+			CellView[] allChangedCellViews = getMapping(allChangedCells.toArray());
+			Rectangle2D changedBounds = getBounds(allChangedCellViews);
+			dirtyRegion = RectUtils.union(dirtyRegion, changedBounds);
+			if (attributes != null) {
+				attributes = handleAttributes(attributes);
+			}
 			// Refresh all changed cells
 			refresh(getMapping(changedCells.toArray(), false), false);
 			// Updates the connected edges. Make sure that changedCells
 			// contains no edges, as these will be removed from the result.
-			Set ctx = DefaultGraphModel.getEdges(getModel(), changedCells
-					.toArray());
-			context = getMapping(ctx.toArray());
 			refresh(context, false);
 			tmp = cells;
 			cells = previousCells;
@@ -2227,15 +2284,17 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 						Rectangle2D src = GraphConstants.getBounds(parent
 								.getAllAttributes());
 						Rectangle2D rect = parent.getBounds();
-						double dx = src.getX() - rect.getX();
-						double dy = src.getY() - rect.getY();
+						if (rect != null && src != null) {
+							double dx = src.getX() - rect.getX();
+							double dy = src.getY() - rect.getY();
 
-						// Gets the attributes from the cell view or
-						// cell and translates the bounds or points
-						AttributeMap attrs = view.getAttributes();
-						if (!attrs.contains(GraphConstants.BOUNDS))
-							attrs = model.getAttributes(view.getCell());
-						attrs.translate(dx, dy);
+							// Gets the attributes from the cell view or
+							// cell and translates the bounds or points
+							AttributeMap attrs = view.getAttributes();
+							if (!attrs.contains(GraphConstants.BOUNDS))
+								attrs = model.getAttributes(view.getCell());
+							attrs.translate(dx, dy);
+						}
 					}
 				}
 			}
@@ -2726,6 +2785,14 @@ public class GraphLayoutCache implements CellMapper, Serializable {
 		this.movesChildrenOnExpand = moveChildrenOnExpand;
 	}
 
+	public boolean isShowsInvisibleEditedCells() {
+		return showsInvisibleEditedCells;
+	}
+
+	public void setShowsInvisibleEditedCells(boolean showsInvisibleEditedCells) {
+		this.showsInvisibleEditedCells = showsInvisibleEditedCells;
+	}
+
 	/**
 	 * @return Returns the collapseXScale.
 	 */
