@@ -12,6 +12,7 @@ import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.HeadlessException;
 import java.awt.Image;
 import java.awt.Insets;
 import java.awt.Point;
@@ -37,7 +38,6 @@ import java.awt.geom.AffineTransform;
 import java.awt.geom.Dimension2D;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
-import java.awt.image.BufferedImage;
 import java.awt.image.VolatileImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
@@ -95,6 +95,7 @@ import org.jgraph.graph.GraphTransferHandler;
 import org.jgraph.graph.ParentMap;
 import org.jgraph.graph.PortView;
 import org.jgraph.plaf.GraphUI;
+import org.jgraph.util.RectUtils;
 
 /**
  * The basic L&F for a graph data structure.
@@ -921,7 +922,6 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 			if (offGraphics == null || offscreen == null) {
 				drawGraph(g, clipBounds);
 				paintOverlay(g);
-				graph.setOffscreenValid(false);
 				return;
 			}
 			if (offscreen instanceof VolatileImage) {
@@ -936,23 +936,28 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 					}
 				} while (((VolatileImage) offscreen).contentsLost());
 			}
-			graph.setOffscreenValid(true);
 			g.drawImage(graph.getOffscreen(), 0, 0, graph);
+			
+			// Paints the handle and marquee regardless of the double buffering
+			// and XOR state so that no artifacts for the marquee appear after
+			// scroll
+			//if (!graph.isXorEnabled()) {
+				// Paint Handle
+				if (handle != null) {
+					handle.paint(g);
+				}
+
+				// Paint Marquee
+				if (marquee != null) {
+					marquee.paint(graph, g);
+				}
+			//}
 			paintOverlay(g);
 			offGraphics.setClip(null);
 		} else {
+			// Not double buffered
 			drawGraph(g, clipBounds);
 		}
-
-	}
-
-	private void renderOffscreen(Graphics offGraphics, Rectangle2D clipBounds) {
-		offGraphics.setColor(graph.getBackground());
-		offGraphics.setPaintMode();
-		Rectangle2D graphBounds = graph.getBounds();
-		offGraphics.fillRect(0, 0, (int) graphBounds.getWidth(), (int) graphBounds
-				.getHeight());
-		drawGraph(offGraphics, clipBounds);
 	}
 
 	/**
@@ -974,14 +979,14 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 	 *            the bounds within graph cells must intersect to be redrawn
 	 */
 	public void drawGraph(Graphics g, Rectangle2D clipBounds) {
+//		if (g.getClip() != null && clipBounds != null && !(g.getClip().equals(clipBounds))) {
+			g.setClip(clipBounds);
+//		}
 		Rectangle2D realClipBounds = null;
 		if (clipBounds != null) {
-			Rectangle2D paintBounds = new Rectangle2D.Double(clipBounds.getX(),
-					clipBounds.getY(), clipBounds.getWidth(), clipBounds
-							.getHeight());
 			realClipBounds = graph.fromScreen(new Rectangle2D.Double(
-					paintBounds.getX(), paintBounds.getY(), paintBounds
-							.getWidth(), paintBounds.getHeight()));
+					clipBounds.getX(), clipBounds.getY(), clipBounds
+							.getWidth(), clipBounds.getHeight()));
 		}
 		// Paint Background (Typically Grid)
 		paintBackground(g);
@@ -1085,9 +1090,10 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 	 * Paint the background of this graph. Calls paintGrid.
 	 */
 	protected void paintBackground(Graphics g) {
-		paintBackgroundImage(g);
+		Rectangle clip = g.getClipBounds();
+		paintBackgroundImage(g, clip);
 		if (graph.isGridVisible()) {
-			paintGrid(graph.getGridSize(), g, g.getClipBounds());
+			paintGrid(graph.getGridSize(), g, clip);
 		}
 	}
 
@@ -1096,11 +1102,13 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 	 * 
 	 * @param g
 	 *            The graphics object to paint the image on.
+	 * @param clip
+	 *            The clipping region to draw into
 	 */
-	protected void paintBackgroundImage(Graphics g) {
+	protected void paintBackgroundImage(Graphics g, Rectangle clip) {
 		Component component = graph.getBackgroundComponent();
 		if (component != null) {
-			paintBackgroundComponent(g, component);
+			paintBackgroundComponent(g, component, clip);
 		}
 		ImageIcon icon = graph.getBackgroundImage();
 		if (icon == null) {
@@ -1141,6 +1149,29 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 		} catch (Error e) {
 		}
 	}
+	
+	/**
+	 * Requests that the component responsible for painting the background paint
+	 * itself
+	 * 
+	 * @param g
+	 *            The graphics object to paint the image on.
+	 * @param component
+	 *            the component to be painted onto the background image
+	 * @param clip
+	 *            The clipping region to draw into
+	 */
+	protected void paintBackgroundComponent(Graphics g, Component component,
+			Rectangle clip) {
+		try {
+			g.setPaintMode();
+			Dimension dim = component.getPreferredSize();
+			rendererPane.paintComponent(g, component, graph, 0, 0, (int) dim
+					.getWidth(), (int) dim.getHeight(), true);
+		} catch (Exception e) {
+		} catch (Error e) {
+		}
+	}
 
 	/**
 	 * Paint the grid.
@@ -1233,7 +1264,7 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 	 * Paint the foreground of this graph. Calls paintPorts.
 	 */
 	protected void paintForeground(Graphics g) {
-		if (graph.isPortsVisible())
+		if (graph.isPortsVisible() && graph.isPortsOnTop())
 			paintPorts(g, graphLayoutCache.getPorts());
 	}
 
@@ -1294,8 +1325,10 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 		if (context != null && !context.isEmpty() && graph.isEnabled()) {
 			try {
 				return new RootHandle(context);
-			} catch (NullPointerException e) {
-				// ignore for now...
+			} catch (HeadlessException e) {
+				// Assume because of running on a server
+			} catch (RuntimeException e) {
+				throw e;
 			}
 		}
 		return null;
@@ -1692,8 +1725,15 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 				// Remove from selection
 				graph.getSelectionModel().removeSelectionCells(removed);
 			}
-			if (graphLayoutCache != null)
+			Rectangle2D oldDirty = null;
+			Rectangle2D dirtyRegion = e.getChange().getDirtyRegion();
+			if (dirtyRegion == null) {
+				oldDirty = graph.getClipRectangle(e.getChange());
+			}
+			
+			if (graphLayoutCache != null) {
 				graphLayoutCache.graphChanged(e.getChange());
+			}
 			// Get arrays
 			Object[] inserted = e.getChange().getInserted();
 			Object[] changed = e.getChange().getChanged();
@@ -1710,6 +1750,15 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 					graph.updateAutoSize(graphLayoutCache.getMapping(
 							changed[i], false));
 			}
+			if (dirtyRegion == null) {
+				Rectangle2D newDirtyRegion = graph.getClipRectangle(e.getChange());
+				dirtyRegion = RectUtils.union(oldDirty, newDirtyRegion);
+				e.getChange().setDirtyRegion(dirtyRegion);
+			}
+			
+			if (dirtyRegion != null) {
+				graph.addOffscreenDirty(dirtyRegion);
+			}
 			// Select if not partial
 			if (!graphLayoutCache.isPartial()
 					&& graphLayoutCache.isSelectsAllInsertedCells()
@@ -1724,7 +1773,7 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 			}
 			updateSize();
 		}
-
+		
 	} // End of BasicGraphUI.GraphModelHandler
 
 	/**
@@ -1746,6 +1795,10 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 							changed[i], false));
 				}
 			}
+			Rectangle2D oldDirtyRegion = e.getChange().getDirtyRegion();
+			graph.addOffscreenDirty(oldDirtyRegion);
+			Rectangle2D newDirtyRegion = graph.getClipRectangle(e.getChange());
+			graph.addOffscreenDirty(newDirtyRegion);
 			Object[] inserted = e.getChange().getInserted();
 			if (inserted != null
 					&& inserted.length > 0
@@ -1798,15 +1851,20 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 						r = lastFocus.getBounds();
 				}
 				if (r != null) {
-					int hsize = (int) (graph.getHandleSize() * graph.getScale()) + 1;
+					Rectangle2D unscaledDirty = graph.fromScreen((Rectangle2D)r.clone());
+					graph.addOffscreenDirty(unscaledDirty);
+					int hsize = (int) graph.getHandleSize() + 1;
 					updateHandle();
-					graph.repaint((int) r.getX() - hsize, (int) r.getY()
-							- hsize, (int) r.getWidth() + 2 * hsize, (int) r
-							.getHeight()
-							+ 2 * hsize);
+					Rectangle dirtyRegion = new Rectangle((int)(r.getX()
+							- hsize), (int)(r.getY() - hsize),
+							(int)(r.getWidth() + 2 * hsize), (int)(r.getHeight() + 2 * hsize));
+					graph.repaint(dirtyRegion);
 				}
-			} else
+			} else {
+				Rectangle dirtyRegion = new Rectangle(graph.getSize());
+				graph.addOffscreenDirty(dirtyRegion);
 				graph.repaint();
+			}
 		}
 	}
 
@@ -2025,10 +2083,11 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 					if (!e.isConsumed() && cell != null) {
 						Object tmp = cell.getCell();
 						boolean wasSelected = graph.isCellSelected(tmp);
-						// if (!wasSelected || e.getModifiers() != 0)
-						selectCellForEvent(tmp, e);
-						focus = cell;
-						postProcessSelection(e, tmp, wasSelected);
+						if (!e.isPopupTrigger() || !wasSelected) {
+							selectCellForEvent(tmp, e);
+							focus = cell;
+							postProcessSelection(e, tmp, wasSelected);
+						}
 					}
 				}
 			} finally {
@@ -2161,9 +2220,10 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 							.getCells()));
 				} else
 					cachedBounds = tmpBounds;
-				if (initialLocation == null)
+				if (initialLocation == null && tmpBounds != null) {
 					initialLocation = new Point2D.Double(tmpBounds.getX(),
 							tmpBounds.getY());
+				}
 				// Sub-Handles
 				Object[] cells = ctx.getCells();
 				if (cells.length < MAXHANDLES) {
@@ -2765,6 +2825,11 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 							double dx = snapCurrent.getX() - start.getX();
 							double dy = snapCurrent.getY() - start.getY();
 							
+							if (!graph.isMoveBelowZero() && initialLocation.getX() + dx < 0)
+							    dx = -1 * initialLocation.getX();
+							if (!graph.isMoveBelowZero() && initialLocation.getY() + dy < 0)
+							    dy = -1 * initialLocation.getY();
+							
 							Point2D tmp = graph.fromScreen(new Point2D.Double(
 									dx, dy));
 							GraphLayoutCache.translateViews(views, tmp.getX(),
@@ -2888,8 +2953,11 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 						|| changeName.equals(JGraph.PORTS_VISIBLE_PROPERTY)
 						|| changeName.equals(JGraph.ANTIALIASED_PROPERTY))
 					graph.repaint();
-				else if (changeName.equals(JGraph.SCALE_PROPERTY))
+				else if (changeName.equals(JGraph.SCALE_PROPERTY)) {
 					updateSize();
+				} else if (changeName.equals(JGraph.PROPERTY_BACKGROUNDIMAGE)) {
+					updateSize();
+				}
 				else if (changeName.equals("font")) {
 					completeEditing();
 					updateSize();
@@ -2947,7 +3015,7 @@ public class BasicGraphUI extends GraphUI implements Serializable {
 		}
 
 		public boolean isEnabled() {
-			return (graph != null && graph.isEnabled());
+			return (graph != null && graph.isEnabled() && graph.isEditing());
 		}
 	} // End of class BasicGraphUI.GraphCancelEditingAction
 
